﻿define("epi-languagemanager/component/command/DeleteLanguageBranch", [
// Dojo base
    "dojo/_base/declare",
    "dojo/_base/lang",

    // EPi CMS
    "epi-cms/ApplicationSettings",

    // Language Manager
    "epi-languagemanager/component/command/CommandBase",

    // Resource
    "epi/i18n!epi/cms/nls/languagemanager.gadget"
],
function (
// Dojo base
    declare,
    lang,

    // EPi CMS
    ApplicationSettings,

    // Language Manager
    CommandBase,

    // Resource
    res
) {

    return declare([CommandBase], {

        // label: [public] String
        //      The action text of the command to be used in visual elements.
        label: res.deletelanguagebranch,

        // category: [readonly] String
        //      A category which provides a hint about how the command could be displayed.
        category: "context",

        postscript: function () {
            this.inherited(arguments);

            this.set("canExecute", false);
        },

        _onModelChange: function () {
            // summary:
            //      Updates canExecute and isAvailable after the model has been updated.
            // tags:
            //      protected

            var item = this.model.currentItemData;

            if (!item) {
                this.set("canExecute", false);
                this.set("isAvailable", false);
                return;
            }

            var isActive = item.isActive && this.model.context.capabilities.deleteLanguageBranch, // check deleteLanguageBranch capability.
                // Do nothing (command disabled) if the content is Wastebasket or Root or has been deleted
                disabled = this.isWastebasket(this.model.contentData) || this.isRoot(this.model.contentData) || this.model.contentData.isDeleted;

            this.set("canExecute",
                item.canDelete
                && item.isCreated
                && !item.isMaster
                && isActive
                && !disabled
                && !ApplicationSettings.disableVersionDeletion
            );

            this.set("isAvailable", item.canDelete && item.isCreated && !item.isMaster && isActive && !ApplicationSettings.disableVersionDeletion);

            this.set("label", lang.replace(res.deletelanguagebranch, item));
        },

        _execute: function () {
            // summary:
            //      Delete a language branch.
            // tags:
            //      protected

            return this.model.deleteLanguageBranch();
        }

    });
});
